#
# Barpanel
# 
# Copyright (C) 2008 Job and Esther Technologies Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

name = "Barpanel"
version = "0.6.0"
copyright = "Copyright (C) 2005 - 2006, Federico Pelloni\nCopyright (C) 2008 Job and Esther Technologies Ltd"
comments = "A powerful themable and extensible panel for your desktop"
website = "http://barpanel.ossproject.org"
authors = [ "Federico Pelloni <federico.pelloni@gmail.com>\nJob and Esther Technologies Ltd" ]
artists = [ "Federico Pelloni <federico.pelloni@gmail.com>\nJob and Esther Technologies Ltd" ]

import os, os.path, gtk, datetime, sys, locale, traceback, pango, shutil

themepaths = [ os.path.realpath(os.path.join(os.path.dirname(__file__), "themes")),
               os.path.expanduser("~/.barpanel/themes") ]
defaultconfig = os.path.join(os.path.dirname(__file__), "config.xml.in")
userconfig = os.path.expanduser("~/.barpanel/config.xml")

tooltips = gtk.Tooltips()
tooltips.enable()

def log(level, message):
    sys.stderr.write("Barpanel [%s] [%-7s] %s\n" % (datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), level, message.replace("\n", " ")))
    sys.stderr.flush()

def logINFO(message):
    log("INFO", message)

def logERROR(message):
    log("ERROR", message)

def logWARNING(message):
    log("WARNING", message)

def logDEBUG(message):
    log("DEBUG", message)

class PluginObject:
    name = None
    module = None
    widget = None
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

class PluginManager:
    def __init__( self, gui ):
        if gui.paneltype == "top" or gui.paneltype == "bottom":
            self.box = gtk.HBox(False, 0)
        else:
            self.box = gtk.VBox(False, 0)
        self.panel = gui
        self.plugins = []
        gui.add(self.box)
        self.box.set_name( "PluginManager" )
        self.box.show_all()

    def load_plugin(self, p, settings, position = -1):
        plugwidget = None
        try:
            logINFO(_("Loading plugin '%s' ..") % (p))
            exec("import barpanel.plugins.%s as plugin" % p)
            if settings == None:
                settings = {}
            settings['pluginname'] = plugin.name
            plugwidget = plugin.Plugin(self.panel, settings)
            expand = plugwidget.getExpand()
            #expand = getattr(plugin, "expand", False)
            self.box.pack_start(plugwidget, expand, True)
            #pobject = PluginObject(name      = plugin.name, 
            #                       module    = plugin, 
            #                       widget    = plugwidget)
            self.plugins.append(plugwidget)
            plugwidget.show()
            #pobject.widget.show()
        except Exception, e:
            logWARNING(_("Unable to load plugin '%s': %s") % (p, e))
            traceback.print_exc()
        return plugwidget

class Plugin(gtk.HBox):
    def __init__(self, panel, settings):
        gtk.HBox.__init__(self, False, 3)
        self.settings = settings
        if settings.has_key('pluginname'):
            self.pluginname = settings['pluginname']
        if self.pluginname==None or len(self.pluginname)<1:
            self.pluginname = "barpanel.Plugin"
        self.panel = panel
        if self.settings.has_key("size"):
            if self.panel.paneltype == "bottom" or self.panel.paneltype == "top":
                self.set_size_request(int(self.settings['size']), -1)
            else:
                self.set_size_request(-1, int(self.settings['size']))
        self.show()
    def logINFO(self, message):
        logINFO("%s: %s" % (self.pluginname, message))
    def logERROR(self, message):
        logERROR("%s: %s" % (self.pluginname, message))
    def logWARNING(self, message):
        logWARNING("%s: %s" % (self.pluginname, message))
    def logDEBUG(self, message):
        logDEBUG("%s: %s" % (self.pluginname, message))
    def getExpand(self):
        v = False
        if self.settings.has_key("expand") and self.settings['expand'] == "yes":
            v = True
        return(v)
    def resize(self):
        pass

class BarpanelWindow(gtk.Window):
    def __init__(self, wtype = gtk.WINDOW_TOPLEVEL):
        gtk.Window.__init__(self, wtype)
        self.set_decorated(False)
        self.set_skip_taskbar_hint(True)
        self.set_skip_pager_hint(True)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)

class PopupWindow(BarpanelWindow):
    def __init__(self, panel, border = 3, wtype = gtk.WINDOW_POPUP):
        BarpanelWindow.__init__(self, wtype)
        self.panel = panel
        self.set_name("BarpanelPopupWindow")
        self.set_border_width(border)
        self.opener = None
        self._box = gtk.VBox()
        self._box.show()
        self._box.set_spacing(3)
        gtk.Window.add(self, self._box)
    
    def add(self, widget, expand = False):
        self._box.pack_start(widget, expand, expand)
    
    def set_header(self, title = None, description = None, icon = None):
        frame = gtk.Frame()
        frame.show()
        self.add(frame)
        self.header = gtk.HBox(False, 10)
        self.header.set_name("BarpanelPopupHeader")
        self.header.show()
        self.header.set_border_width(3)
        frame.add(self.header)
        if icon:
            if type(icon) == gtk.gdk.Pixbuf:
                image = gtk.Image()
                image.set_from_pixbuf(icon)
            elif type(icon) == gtk.Image:
                image = icon
            image.show()
            self.header.pack_start(image, False, False)
        if title or description:
            lbl = gtk.Label("")
            txt = "<big>"
            if title:
                txt += "<b>%s</b>" % title
                if description:
                    txt += "\n"
            if description:
                txt += description
            txt += "</big>"
            lbl.set_markup(txt)
            lbl.set_property("xalign", 0)
            lbl.show()
            self.header.add(lbl)
    
    def run(self):
        self.show()
    
    def reposition(self):
        window_x, window_y = self.panel.get_position()
        window_w, window_h = self.panel.get_size()
        x = 0
        y = 0
        if self.panel.paneltype == "top" or self.panel.paneltype == "bottom":
            x = self.x + window_x
            h = self.get_size()[1]
            if self.panel.paneltype == "top":
                y = window_y + window_h
            elif self.panel.paneltype == "bottom":
                y = window_y - h
            myw = self.get_size()[0]
            while x >= 0 and x + myw > gtk.gdk.screen_width():
                x = x - 1
        else:
            y = self.y + window_y
            w = self.get_size()[0]
            if self.panel.paneltype == "left":
                x = window_x + window_w
            elif self.panel.paneltype == "right":
                x = window_x - w
            myh = self.get_size()[1]
            while y >= 0 and y + myh > gtk.gdk.screen_height():
                y = y - 1
        self.move(x, y) 
        
    def open(self, widget = 0):
        if self.panel.opened_popup:
            if id(self) == id(self.panel.opened_popup):
                return
            self.panel.opened_popup.close()
        if type(widget) == type(1):
            self.x = widget
            self.y = widget
        else:
            self.opener = widget
            try:
                self.x = widget.allocation.x
                self.y = widget.allocation.y
            except:
                self.x = 0
                self.y = 0
                self.opener = None
        self.reposition()
        self.show()
        self.panel.opened_popup = self
        if self.panel.autohide == "yes":
            self.panel.resize()
        
    def close(self):
        self.hide()
        try:
            self.opener.set_active(False)
        except:
            raise
        self.panel.opened_popup = None
        if self.panel.autohide == "yes":
            self.panel.resize()
    
    def toggle(self, widget = 0):
        if self.panel.opened_popup:
            if id(self) != id(self.panel.opened_popup):
                self.panel.opened_popup.close()
                self.open(widget)
            else:
                self.close()
        else:
            self.open(widget)

    def toggleConditional(self, a, b):
        pass

class BarpanelMenuWindow(PopupWindow):
    __is_size_set = False
    def __init__(self, name, description, icon, panel, border = 3, wtype = gtk.WINDOW_POPUP):
        PopupWindow.__init__(self, panel, border, wtype)
        self.set_header(name, description, icon)
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_NONE)
        self.scroll.show()
        self.add(self.scroll, True)
        self.buttons = gtk.VBox(True, 0)
        self.buttons.show()
        self.scroll.add_with_viewport(self.buttons)

    def append(self, obj):
        self.buttons.add(obj)
        obj.show()
        
    def toggle(self, widget = 0):
        if not self.__is_size_set:
            w, h = self.buttons.size_request()
            pw, ph = self.panel.get_size()
            self.scroll.set_size_request(w, min(h + 5, gtk.gdk.screen_height()*8/10)) # - int(ph) - 200))
            self.__is_size_set = True
        self.scroll.get_vadjustment().set_value(0)
        PopupWindow.toggle(self, widget)

def launch_command(cmd):
    logINFO(_("Barpanel: launching '%s'") % (cmd))
    os.spawnl(os.P_NOWAIT, "/bin/sh", "/bin/sh", "-c", cmd)

class Panel(BarpanelWindow):
    def __init__(self, paneltype, align, span, size, autohide, iconsize, fontdesc, mgr):
        BarpanelWindow.__init__( self )
        #self.set_keep_above(True)
        self.stick()
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
        self.opened_popup = None
        self.set_title( "Barpanel" )
        self.set_name( "BarpanelWindow" )
        self.align = align
        self.paneltype = paneltype
        self.span = span
        self.autohide = autohide
        if self.span == None or len(self.span)<1:
            self.span = "100%"
        self.size = locale.atoi(size)
        self.plugin_manager = PluginManager(self)
        mgr.registerPanel(self)
        self.connect("configure-event", self.onConfigureEvent)
        self.panelmanager = mgr
        self.lastw = 0
        self.lasth = 0
        self.iconsize = locale.atoi(iconsize)
        self.fontdesc = fontdesc
        self.hasmouse = False
        self.connect("enter-notify-event", self.onMouseEnter)
        self.connect("leave-notify-event", self.onMouseLeave)

    def onMouseEnter(self, widget, event):
        if self.hasmouse == False:
            self.hasmouse = True
            self.displayPanel()

    def onMouseLeave(self, widget, event):
        if self.hasmouse == True and event.detail != gtk.gdk.NOTIFY_INFERIOR: # 2
            self.hasmouse = False
            self.displayPanel()

    def update_font(self, plugin, child):
        if self.fontdesc!=None and len(self.fontdesc)>0:
            child.modify_font(pango.FontDescription(self.fontdesc))

    def get_image_from_icon_name(self, icon, pixelsize=-1):
        img = gtk.image_new_from_icon_name(icon, gtk.ICON_SIZE_BUTTON)
        if pixelsize < 0:
            img.set_pixel_size(self.iconsize)
        return(img)

    def get_stock_image(self, stockid, pixelsize=-1):
        img = gtk.image_new_from_stock(stockid, gtk.ICON_SIZE_BUTTON)
        if pixelsize < 0:
            img.set_pixel_size(self.iconsize)
        return(img)

    def update_gtk_image(self, image, file, pixelsize=-1):
        #image.set_from_file("%s/images/%s.png" % (os.path.realpath(os.path.dirname(__file__)), file))
        if file[0] == '/':
            path = file
        else:
            path = "%s/images/%s.png" % (os.path.realpath(os.path.dirname(__file__)), file)
        pixbuf = gtk.gdk.pixbuf_new_from_file(path)
        if pixelsize < 0:
            pixelsize = self.iconsize
        bw = pixbuf.get_width()
        bh = pixbuf.get_height()
        if bh!=pixelsize: # or bw!=pixelsize
            nw = pixelsize * bw / bh
            scaled_buf = pixbuf.scale_simple(nw,pixelsize,gtk.gdk.INTERP_BILINEAR)
            image.set_from_pixbuf(scaled_buf)
        else:
            image.set_from_pixbuf(pixbuf)

    def get_image_from_file(self, file, pixelsize=-1):
        v = gtk.Image()
        self.update_gtk_image(v, file, pixelsize)
        return(v)

    def onConfigureEvent(self, window, event):
        if event.width != self.lastw or event.height != self.lasth:
            self.lastw = event.width
            self.lasth = event.height
            #self.strut_set()
            self.reposition()

    def available_width(self):
        w = gtk.gdk.screen_width()
        for p in self.panelmanager.panels:
            if id(p) == id(self):
                break
            if (p.paneltype == "left" or p.paneltype == "right") and p.autohide == "no":
                w -= p.get_size()[0]
        return(w)

    def available_height(self):
        h = gtk.gdk.screen_height()
        for p in self.panelmanager.panels:
            if id(p) == id(self):
                break
            if (p.paneltype == "top" or p.paneltype == "bottom") and p.autohide == "no":
                h -= p.get_size()[1]
        return(h)

    def x_offset(self):
        x = 0
        for p in self.panelmanager.panels:
            if id(p) == id(self):
                break
            if p.paneltype == "left" and p.autohide == "no":
                x += p.get_size()[0]
        return(x)

    def y_offset(self):
        y = 0
        for p in self.panelmanager.panels:
            if id(p) == id(self):
                break
            if p.paneltype == "top" and p.autohide == "no":
                y += p.get_size()[1]
        return(y)

    def resize(self):
        if self.paneltype == "bottom" or self.paneltype == "top":
            w = self.available_width()
            if self.span.find("%") > 0:
                nw = locale.atoi(self.span.replace("%", "")) * w / 100
            else:
                nw = locale.atoi(self.span)
            if nw < w:
                w = nw
            size = self.size
            if self.autohide == "yes" and self.hasmouse == False and self.opened_popup == None:
                size = 1
            BarpanelWindow.resize(self, w, size)
            self.set_size_request(w, size)
        elif self.paneltype == "right" or self.paneltype == "left":
            h = self.available_height()
            if self.span.find("%") > 0:
                nh = locale.atoi(self.span.replace("%", "")) * h / 100
            else:
                nh = locale.atoi(self.span)
            if nh < h:
                h = nh
            size = self.size
            if self.autohide == "yes" and self.hasmouse == False and self.opened_popup == None:
                size = 1
            BarpanelWindow.resize(self, size, h)
            self.set_size_request(size, h)

    def strut_set(self):
        if self.window:
            w, h = self.get_size()
            x, y = self.get_position()
            if self.autohide == "yes":
                if self.paneltype == "bottom" or self.paneltype == "top":
                    h = 1
                else:
                    w = 1
            if self.paneltype == "bottom":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [0,0,0,h])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [0,0,0,h, 0,0,0,0, 0,0,0,w])
            elif self.paneltype == "top":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [0,0,h,0])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [0,0,h,0, 0,0,0,0, 0,w,0,0])
            elif self.paneltype == "left":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [w,0,0,0])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [w,0,0,0, 0,h,0,0, 0,0,0,0])
            elif self.paneltype == "right":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [0,w,0,0])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [0,w,0,0, 0,0,0,h, 0,0,0,0])

    def reposition( self ):
        x = self.x_offset()
        y = self.y_offset()
        width, height = self.get_size()
        if self.paneltype == "top":
            x = self.x_offset()
            y = self.y_offset()
        elif self.paneltype == "bottom":
            x = self.x_offset()
            y = self.y_offset() + self.available_height() - height
        elif self.paneltype == "left":
            x = self.x_offset()
            y = self.y_offset()
        elif self.paneltype == "right":
            x = self.x_offset() + self.available_width() - width
            y = self.y_offset()
        if self.paneltype == "top" or self.paneltype == "bottom":
            if self.align == "left":
                x = self.x_offset()
            elif self.align == "right":
                x = self.x_offset() + self.available_width() - width
            elif self.align == "center":
                x = self.x_offset() + (self.available_width() - width) / 2
        else:
            if self.align == "top":
                y = self.y_offset()
            elif self.align == "bottom":
                y = self.y_offset() + self.available_height() - height
            elif self.align == "center":
                y = self.y_offset() + (self.available_height() - height) / 2
        self.move( x, y )
        if self.opened_popup!=None:
            self.opened_popup.reposition()
        self.strut_set()

    def displayPanel(self):
        self.resize()
        self.show_all()
        #self.reposition()
        #for p in self.plugin_manager.plugins:
        #    p.resize()

def load_theme(theme):
    try:
        logINFO(_("Loading theme '%s' .." % theme))
        path = None
        for p in themepaths:
            try:
                path = os.path.join(p, theme, "gtkrc")
                if os.path.isfile(path):
                    break
                else:
                    path = None
            except:
                path = None
        if path is None or not os.path.isfile(path):
            raise
        gtk.rc_reset_styles(gtk.settings_get_default())
        gtk.rc_parse(path)
    except:
        logINFO(_("Warning: unable to load theme %s, using default" % theme))

from xml.sax import handler, make_parser
from xml.sax.handler import feature_namespaces
class XMLConfig(handler.EntityResolver, handler.DTDHandler, handler.ContentHandler, handler.ErrorHandler):
    def __init__(self, mgr):
        self.panel = None
        self.plugin = None
        self.config = None
        self.item = None
        self.mgr = mgr

    def startElement(self, name, attrs):
        if name == "barpanel":
            theme = attrs.get("theme", None)
            if theme!=None and len(theme)>0:
                load_theme(theme)
        elif name == "panel":
            paneltype = attrs.get("type", None)
            if paneltype == None:
                paneltype = attrs.get("pos", None)
            if paneltype!=None and len(paneltype)>0:
                self.panel = Panel(paneltype, attrs.get("align", "center"), attrs.get("span", "100%"), attrs.get("size", "32"), attrs.get("autohide", "no"), attrs.get("iconsize", "24"), attrs.get("font", ""), self.mgr)
        elif name == "plugin":
            pname = attrs.get("name", None)
            if pname!=None and len(pname)>0:
                self.plugin = {}
                self.plugin['name'] = pname
        elif name == "config" and self.plugin!=None:
            cname = attrs.get("name", None)
            if cname!=None and len(cname)>0:
                self.config = {}
                self.config['name'] = cname
                self.config['value'] = ""
        elif name == "item" and self.plugin!=None:
            self.item = {}
        elif name == "desktop":
            bg = attrs.get("background", None)
            style = attrs.get("style", "tile")
            if bg != None:
                logINFO("Setting desktop background to '%s' .." % bg)
                rootwindow = gtk.gdk.get_default_root_window()
                if bg[0] == "#": 
                   rootwindow.set_background(gtk.gdk.color_parse(bg))
                else:
                    try:
                        pixbuf = gtk.gdk.pixbuf_new_from_file(bg)
                        if style == "scale":
                            ws, wh = rootwindow.get_size()
                            pixbuf = pixbuf.scale_simple(ws, wh, gtk.gdk.INTERP_BILINEAR)
                        pixmap, mask = pixbuf.render_pixmap_and_mask()
                        #width, height = pixmap.get_size()
                        del pixbuf
                        rootwindow.set_back_pixmap(pixmap, False)
                        del pixmap
                    except:
                        logERROR("Failed to set desktop background image")
                rootwindow.clear()

    def characters(self, ch):
        if self.config!=None:
            self.config['value'] = self.config['value'] + ch

    def endElement(self, name):
        if name == "panel" and self.panel!=None:
            self.panel.displayPanel()
            self.panel = None
        elif name == "plugin" and self.plugin!=None and self.panel!=None:
            if self.plugin.has_key("config"):
                self.panel.plugin_manager.load_plugin(
                    self.plugin['name'],
                    self.plugin['config'])
            else:
                self.panel.plugin_manager.load_plugin(self.plugin['name'], None)
        elif name == "config" and self.config!=None and self.item!=None:
            self.item[self.config['name']] = self.config['value']
            self.config = None
        elif name == "config" and self.config!=None and self.plugin!=None:
            if self.plugin.has_key("config") == False:
                self.plugin['config'] = {}
            self.plugin['config'][self.config['name']] = self.config['value']
            self.config = None
        elif name == "item" and self.item!=None and self.plugin!=None:
            inum = 1
            while self.plugin['config'].has_key("item%d" % (inum)):
                inum = inum + 1
            self.plugin['config']["item%d" % (inum)] = self.item
            self.item = None

    def parseFile(self, file):
        logINFO(_("Reading configuration file '%s' .." % (file)))
        parser = make_parser()
        parser.setFeature(feature_namespaces, 0)
        parser.setContentHandler(self)
        parser.parse(file)

class PanelManager:
    def __init__(self):
        self.panels = []

    def registerPanel(self, panel):
        self.panels.append(panel)

    def destroyPanels(self):
        while len(self.panels) > 0:
            p = self.panels.pop()
            p.destroy()

    def redraw_and_reposition(self):
        for p in self.panels:
            p.displayPanel()

    def reload(self, forcedefault = False):
        self.destroyPanels()
        config = XMLConfig(self)
        if forcedefault == True:
            logINFO("Default configuration forced: removing personal configuration file.")
            try:
                os.unlink(userconfig)
            except:
                logERROR("Failed to remove personal configuration file")
        if os.path.exists(userconfig) == False:
            try:
                logINFO(_("No user configuration file '%s' exists. Creating one using the default config." % (userconfig)))
                shutil.copyfile(defaultconfig, userconfig)
            except Exception, e:
                logERROR(str(e))
        if os.path.exists(userconfig):
            config.parseFile(userconfig)
        else:
            config.parseFile(defaultconfig)
